@@ -298,30 +298,13 @@ protected:
     /// @return Returns a ConfigPtr to the new context instance.
     virtual process::ConfigPtr createNewContext();
 
-    /// @brief Redact the configuration.
+    /// @brief Return a list of all paths that contain passwords or secrets.
     ///
-    /// This method replaces passwords by asterisks. The agent version
-    /// applies authentication clients.
+    /// Used in @ref isc::process::DCfgMgrBase::redactConfig.
     ///
-    /// @param config Pointer to a configuration specified for the agent.
-    /// @return unmodified config or a copy of the config where passwords were
-    /// replaced by asterisks so can be safely logged to an unprivileged place.
-    virtual isc::data::ConstElementPtr
-    redactConfig(isc::data::ConstElementPtr config) const;
-
-private:
-    /// @brief Redact an element.
-    ///
-    /// Recursive helper of redactConfig.
-    ///
-    /// @param elem An element to redact.
-    /// @param redacted The reference to redacted flag: true means the result
-    /// was redacted so cannot be shared.
-    /// @return unmodified element or a copy of the element: in the second
-    /// case embedded passwords were replaced by asterisks and the redacted
-    /// flag was set to true.
-    virtual isc::data::ConstElementPtr
-    redactElement(isc::data::ConstElementPtr elem, bool& redacted) const;
+    /// @return the list of lists of sequential JSON map keys needed to reach
+    /// the passwords and secrets.
+    std::list<std::list<std::string>> jsonPathsToRedact() const final override;
 };
 
 /// @brief Defines a shared pointer to CtrlAgentCfgMgr.
