@@ -73,8 +73,9 @@ public:
     /// used by the server.
     /// @param query Pointer to the client message.
     /// @param subnet Pointer to the subnet to which the client belongs.
+    /// @param drop if it is true the packet will be dropped.
     Dhcpv4Exchange(const AllocEnginePtr& alloc_engine, const Pkt4Ptr& query,
-                   const Subnet4Ptr& subnet);
+                   const Subnet4Ptr& subnet, bool& drop);
 
     /// @brief Initializes the instance of the response message.
     ///
@@ -1160,6 +1161,13 @@ public:
     /// @brief Returns the index for "lease4_decline" hook point
     /// @return the index for "lease4_decline" hook point
     static int getHookIndexLease4Decline();
+
+    /// @brief Return a list of all paths that contain passwords or secrets for
+    /// kea-dhcp4.
+    ///
+    /// @return the list of lists of sequential JSON map keys needed to reach
+    /// the passwords and secrets.
+    std::list<std::list<std::string>> jsonPathsToRedact() const final override;
 };
 
 }  // namespace dhcp
