@@ -126,8 +126,8 @@ CommandOptions::reset() {
     addr_unique_ = false;
     mac_list_file_.clear();
     mac_list_.clear();
-    giaddr_list_file_.clear();
-    giaddr_list_.clear();
+    relay_addr_list_file_.clear();
+    relay_addr_list_.clear();
     multi_subnet_ = false;
     num_request_.clear();
     exit_wait_time_ = 0;
@@ -390,8 +390,8 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
         case 'J':
             check(num_subnet_list_files >= 1, "only one -J option can be specified");
             num_subnet_list_files++;
-            giaddr_list_file_ = std::string(optarg);
-            loadGiaddr();
+            relay_addr_list_file_ = std::string(optarg);
+            loadRelayAddr();
             break;
 
         case 'l':
@@ -850,27 +850,30 @@ CommandOptions::convertHexString(const std::string& text) const {
 
 bool CommandOptions::validateIP(const std::string& line) {
     try {
-        asiolink::IOAddress ip_address_ = isc::asiolink::IOAddress(line);
-        // let's silence not used warning
-        (void) ip_address_;
+        isc::asiolink::IOAddress ip_address(line);
+        if ((getIpVersion() == 4 && !ip_address.isV4()) ||
+            (getIpVersion() == 6 && !ip_address.isV6())) {
+            return (true);
+        }
     } catch (const isc::asiolink::IOError& e) {
         return (true);
     }
-    giaddr_list_.push_back(line);
+    relay_addr_list_.push_back(line);
     multi_subnet_ = true;
     return (false);
 }
 
-void CommandOptions::loadGiaddr() {
+void CommandOptions::loadRelayAddr() {
     std::string line;
-    std::ifstream infile(giaddr_list_file_.c_str());
+    std::ifstream infile(relay_addr_list_file_.c_str());
     size_t cnt = 0;
     while (std::getline(infile, line)) {
         cnt++;
         stringstream tmp;
-        tmp << "invalid address in line: "<< cnt;
+        tmp << "invalid address or wrong address version in line: "<< cnt;
         check(validateIP(line), tmp.str());
     }
+    check(cnt == 0, "file with addresses is empty!");
 }
 
 void CommandOptions::loadMacs() {
@@ -1237,7 +1240,7 @@ CommandOptions::usage() const {
         "    whether -6 is given.\n"
         "-I<ip-offset>: Offset of the (DHCPv4) IP address in the requested-IP\n"
         "    option / (DHCPv6) IA_NA option in the (second/request) template.\n"
-        "-J<giaddr-list-file>: Text file that include multiple addresses.\n"
+        "-J<remote-address-list-file>: Text file that include multiple addresses.\n"
         "    If provided perfdhcp will choose randomly one of addresses for each\n"
         "    exchange.\n"
         "-l<local-addr|interface>: For DHCPv4 operation, specify the local\n"
