@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -841,17 +841,49 @@ TEST_F(CommandOptionsTest, LoadMacsFromFile) {
     EXPECT_EQ(4, m.size());
 }
 
-TEST_F(CommandOptionsTest, LoadGiaddrFromFile) {
+TEST_F(CommandOptionsTest, LoadRelay4AddrFromFile) {
     CommandOptions opt;
-    std::string giaddr_list_full_path = getFullPath("giaddr-list.txt");
+    std::string relay_addr_list_full_path = getFullPath("relay4-list.txt");
     std::ostringstream cmd;
-    cmd << "perfdhcp -J " << giaddr_list_full_path << " abc";
+    cmd << "perfdhcp -4 -J " << relay_addr_list_full_path << " abc";
     EXPECT_NO_THROW(process(opt, cmd.str()));
-    EXPECT_EQ(giaddr_list_full_path, opt.getGiaddrListFile());
+    EXPECT_EQ(relay_addr_list_full_path, opt.getRelayAddrListFile());
     EXPECT_TRUE(opt.checkMultiSubnet());
-    EXPECT_EQ(5, opt.getGiaddrList().size());
+    EXPECT_EQ(5, opt.getRelayAddrList().size());
 }
 
+TEST_F(CommandOptionsTest, LoadRelay6AddrFromFile) {
+    CommandOptions opt;
+    std::string relay_addr_list_full_path = getFullPath("relay6-list.txt");
+    std::ostringstream cmd;
+    cmd << "perfdhcp -6 -J " << relay_addr_list_full_path << " abc";
+    EXPECT_NO_THROW(process(opt, cmd.str()));
+    EXPECT_EQ(relay_addr_list_full_path, opt.getRelayAddrListFile());
+    EXPECT_TRUE(opt.checkMultiSubnet());
+    EXPECT_EQ(2, opt.getRelayAddrList().size());
+}
+
+TEST_F(CommandOptionsTest, RelayAddr6ForVersion4) {
+    CommandOptions opt;
+    std::string relay_addr_list_full_path = getFullPath("relay6-list.txt");
+    std::ostringstream cmd;
+    cmd << "perfdhcp -4 -J " << relay_addr_list_full_path << " abc";
+    EXPECT_THROW(process(opt, cmd.str()), isc::InvalidParameter);
+    EXPECT_FALSE(opt.checkMultiSubnet());
+    EXPECT_EQ(0, opt.getRelayAddrList().size());
+}
+
+TEST_F(CommandOptionsTest, RelayAddr4ForVersion6) {
+    CommandOptions opt;
+    std::string relay_addr_list_full_path = getFullPath("relay4-list.txt");
+    std::ostringstream cmd;
+    cmd << "perfdhcp -6 -J " << relay_addr_list_full_path << " abc";
+    EXPECT_THROW(process(opt, cmd.str()), isc::InvalidParameter);
+    EXPECT_FALSE(opt.checkMultiSubnet());
+    EXPECT_EQ(0, opt.getRelayAddrList().size());
+}
+
+
 TEST_F(CommandOptionsTest, LoadMacsFromFileNegativeCases) {
     CommandOptions opt;
     // Negative test cases
