@@ -32,6 +32,12 @@ RunScriptImpl::configure(LibraryHandle& handle) {
     if (name->getType() != Element::string) {
         isc_throw(InvalidParameter, "The 'name' parameter must be a string");
     }
+    IOServicePtr io_service(new asiolink::IOService());
+    try {
+        ProcessSpawn process(io_service, name->stringValue());
+    } catch (const isc::Exception& ex) {
+        isc_throw(InvalidParameter, "Invalid 'name' parameter: " << ex.what());
+    }
     setName(name->stringValue());
     ConstElementPtr sync = handle.getParameter("sync");
     if (sync) {
