@@ -82,7 +82,21 @@ public:
     /// \brief Stop the underlying event loop.
     ///
     /// This will return the control to the caller of the \c run() method.
-    void stop() { io_service_.stop();} ;
+    void stop() {
+        io_service_.stop();
+    }
+
+    /// \brief Indicates if the IOService has been stopped.
+    ///
+    /// \return true if the IOService has been stopped, false otherwise.
+    bool stopped() const {
+        return (io_service_.stopped());
+    }
+
+    /// \brief Restarts the IOService in preparation for a subsequent \c run() invocation.
+    void restart() {
+        io_service_.reset();
+    }
 
     /// \brief Removes IO service work object to let it finish running
     /// when all handlers have been invoked.
@@ -96,7 +110,9 @@ public:
     /// that share the same \c io_service with the authoritative server.
     /// It will eventually be removed once the wrapper interface is
     /// generalized.
-    boost::asio::io_service& get_io_service() { return io_service_; };
+    boost::asio::io_service& get_io_service() {
+        return (io_service_);
+    }
 
     /// \brief Post a callback on the IO service
     ///
@@ -105,6 +121,7 @@ public:
         const CallbackWrapper wrapper(callback);
         io_service_.post(wrapper);
     }
+
 private:
     boost::asio::io_service io_service_;
     boost::shared_ptr<boost::asio::io_service::work> work_;
@@ -136,6 +153,16 @@ IOService::stop() {
     io_impl_->stop();
 }
 
+bool
+IOService::stopped() const {
+    return (io_impl_->stopped());
+}
+
+void
+IOService::restart() {
+    io_impl_->restart();
+}
+
 void
 IOService::stopWork() {
     io_impl_->stopWork();
