@@ -10,6 +10,7 @@
 #include <asiolink/io_address.h>
 #include <asiolink/io_service.h>
 #include <http/listener.h>
+#include <http/http_thread_pool.h>
 #include <thread>
 #include <vector>
 
@@ -26,7 +27,7 @@ namespace config {
 /// configured thread pool size.
 ///
 /// @note This class is NOT compatible with Kea core single-threading.
-/// It is incumbant upon the owner to ensure the Kea core multi-threading
+/// It is incumbent upon the owner to ensure the Kea core multi-threading
 /// is (or will be) enabled when creating instances of this class.
 class CmdHttpListener {
 public:
@@ -37,43 +38,79 @@ public:
     /// @brief Destructor
     virtual ~CmdHttpListener();
 
-    /// @brief Initiates the listener's worker thread.
+    /// @brief Starts running the listener's thread pool.
     void start();
 
-    /// @brief Stops the listener's worker thread.
+    /// @brief Pauses the listener's thread pool.
+    ///
+    /// Suspends thread pool event processing.
+    void pause();
+
+    /// @brief Resumes running the listener's thread pool.
+    ///
+    /// Resumes thread pool event processing.
+    void resume();
+
+    /// @brief Stops the listener's thread pool.
     void stop();
 
-    /// @brief Checks if we are listening to the HTTP requests.
+    /// @brief Indicates if the thread pool is running.
     ///
-    /// @return true if we are listening.
-    bool isListening() const;
+    /// @return True if the thread pool exists and it is in the RUNNING state,
+    /// false otherwise.
+    bool isRunning();
+
+    /// @brief Indicates if the thread pool is stopped.
+    ///
+    /// @return True if the thread pool does not exist or it is in the STOPPED
+    /// state, false otherwise.
+    bool isStopped();
+
+    /// @brief Indicates if the thread pool is paused.
+    ///
+    /// @return True if the thread pool exists and it is in the PAUSED state,
+    /// false otherwise.
+    bool isPaused();
 
     /// @brief Fetches the IP address on which to listen.
     ///
     /// @return IOAddress containing the address on which to listen.
-    isc::asiolink::IOAddress& getAddress() {
+    isc::asiolink::IOAddress getAddress() const {
         return (address_);
     }
 
     /// @brief Fetches the port number on which to listen.
     ///
     /// @return uint16_t containing the port number on which to listen.
-    uint16_t getPort() {
+    uint16_t getPort() const {
         return (port_);
     }
 
     /// @brief Fetches the maximum size of the thread pool.
     ///
     /// @return uint16_t containing the maximum size of the thread pool.
-    uint16_t getThreadPoolSize() {
+    uint16_t getThreadPoolSize() const {
         return (thread_pool_size_);
     }
 
     /// @brief Fetches the number of threads in the pool.
     ///
     /// @return uint16_t containing the number of running threads.
-    uint16_t getThreadCount() {
-        return (threads_.size());
+    uint16_t getThreadCount() const {
+        if (!thread_pool_) {
+            return (0);
+        }
+
+        return (thread_pool_->getThreadCount());
+    }
+
+    /// @brief Fetches a pointer to the internal IOService used to
+    /// drive the thread-pool in multi-threaded mode.
+    ///
+    /// @return pointer to the IOService instance, or an empty pointer
+    /// in single-threaded mode.
+    asiolink::IOServicePtr getThreadIOService() const {
+        return (thread_io_service_);
     }
 
 private:
@@ -84,7 +121,7 @@ private:
     uint16_t port_;
 
     /// @brief IOService instance that drives our IO.
-    isc::asiolink::IOServicePtr io_service_;
+    isc::asiolink::IOServicePtr thread_io_service_;
 
     /// @brief The HttpListener instance
     http::HttpListenerPtr http_listener_;
@@ -93,7 +130,7 @@ private:
     std::size_t thread_pool_size_;
 
     /// @brief The pool of threads that do IO work.
-    std::vector<boost::shared_ptr<std::thread> > threads_;
+    http::HttpThreadPoolPtr thread_pool_;
 };
 
 /// @brief Defines a shared pointer to CmdHttpListener.
