@@ -52,9 +52,10 @@ public:
     /// Starts test timer which detects timeouts, deregisters all commands
     /// from CommandMgr, and enables multi-threading mode.
     CmdHttpListenerTest()
-        : io_service_(), test_timer_(io_service_), run_io_service_timer_(io_service_),
-        clients_(), num_threads_(), num_clients_(), num_in_progress_(0), num_finished_(0),
-        chunk_size_(0) {
+        : listener_(), io_service_(), test_timer_(io_service_),
+          run_io_service_timer_(io_service_), clients_(), num_threads_(),
+          num_clients_(), num_in_progress_(0), num_finished_(0), chunk_size_(0),
+          pause_cnt_(0) {
         test_timer_.setup(std::bind(&CmdHttpListenerTest::timeoutHandler, this, true),
                           TEST_TIMEOUT, IntervalTimer::ONE_SHOT);
 
@@ -69,6 +70,9 @@ public:
     ///
     /// Removes HTTP clients, unregisters commands, disables MT.
     virtual ~CmdHttpListenerTest() {
+        // Wipe out the listener.
+        listener_.reset();
+
         // Destroy all remaining clients.
         for (auto const& client : clients_) {
             client->close();
@@ -171,28 +175,31 @@ public:
     /// @brief Runs IO service with optional timeout.
     ///
     /// We iterate over calls to asio::io_service.run(), until
-    /// all the clients have completed their requests.  We do it this
+    /// all the clients have completed their requests.  We do it this way
     /// because the test clients stop the io_service when they're
     /// through with a request.
     ///
-    /// @param timeout Optional value specifying for how long the io service
-    /// should be ran.
-    void runIOService() {
+    /// @param request_limit Desired number of requests the function should wait
+    /// to be processed before returning.
+    void runIOService(size_t request_limit = 0) {
+        if (!request_limit) {
+            request_limit = clients_.size();
+        }
+
         // Loop until the clients are done, an error occurs, or the time runs out.
-        bool keep_going = true;
-        while (keep_going) {
-            // Always call reset() before we call run();
-            io_service_.get_io_service().reset();
+        size_t num_done = 0;
+        while (num_done != request_limit) {
+            // Always call restart() before we call run();
+            io_service_.restart();
 
             // Run until a client stops the service.
             io_service_.run();
 
             // If all the clients are done receiving, the test is done.
-            keep_going = false;
+            num_done = 0;
             for (auto const& client : clients_) {
-                if (!client->receiveDone()) {
-                    keep_going = true;
-                    break;
+                if (client->receiveDone()) {
+                    ++num_done;
                 }
             }
         }
@@ -246,8 +253,8 @@ public:
     ///
     /// @return Returns response with map of arguments containing
     /// a string value 'thread-id': <thread id>
-    ConstElementPtr threadCommandHandler(const std::string& /*command_name*/,
-                                         const ConstElementPtr& command_arguments) {
+    ConstElementPtr synchronizedCommandHandler(const std::string& /*command_name*/,
+                                               const ConstElementPtr& command_arguments) {
         // If the number of in progress commands is less than the number
         // of threads, then wait here until we're notified.  Otherwise,
         // notify everyone and finish.  The idea is to force each thread
@@ -304,9 +311,39 @@ public:
         return (createAnswer(CONTROL_RESULT_SUCCESS, arguments));
     }
 
+    /// @brief Handler for the 'thread' command.
+    ///
+    /// @param command_name Command name, i.e. 'thread'.
+    /// @param command_arguments Command arguments should contain
+    /// one string element, "client-ptr", whose value is the stringified
+    /// pointer to the client that issued the command.
+    ///
+    /// @return Returns response with map of arguments containing
+    /// a string value 'thread-id': <thread id>
+    ConstElementPtr simpleCommandHandler(const std::string& /*command_name*/,
+                                         const ConstElementPtr& command_arguments) {
+        // Create the map of response arguments.
+        ElementPtr arguments = Element::createMap();
+        // First we echo the client-ptr command argument.
+        ConstElementPtr client_ptr = command_arguments->get("client-ptr");
+        if (!client_ptr) {
+            return (createAnswer(CONTROL_RESULT_ERROR, "missing client-ptr"));
+        }
+
+        arguments->set("client-ptr", client_ptr);
+
+        // Now we add the thread-id.
+        std::stringstream ss;
+        ss << std::this_thread::get_id();
+        arguments->set("thread-id", Element::create(ss.str()));
+
+        // We're done, ship it!
+        return (createAnswer(CONTROL_RESULT_SUCCESS, arguments));
+    }
+
     /// @brief Submits one or more thread commands to a CmdHttpListener.
     ///
-    /// This function command will creates a CmdHttpListener
+    /// This function command will create a CmdHttpListener
     /// with the given number of threads, initiates the given
     /// number of clients, each requesting the "thread" command,
     /// and then iteratively runs the test's IOService until all
@@ -325,8 +362,8 @@ public:
     /// if it is greater than num_threads.
     void threadListenAndRespond(size_t num_threads, size_t num_clients) {
         // First we makes sure the parameter rules apply.
-        ASSERT_TRUE(num_threads > 0);
-        ASSERT_TRUE(num_clients > 0);
+        ASSERT_TRUE(num_threads);
+        ASSERT_TRUE(num_clients);
         ASSERT_TRUE((num_clients < num_threads) || (num_clients % num_threads == 0));
 
         num_threads_ = num_threads;
@@ -339,19 +376,18 @@ public:
         // Register the thread command handler.
         CommandMgr::instance().registerCommand("thread",
                                                std::bind(&CmdHttpListenerTest
-                                                         ::threadCommandHandler,
+                                                         ::synchronizedCommandHandler,
                                                          this, ph::_1, ph::_2));
 
         // Create a listener with prescribed number of threads.
-        CmdHttpListenerPtr listener;
-        ASSERT_NO_THROW_LOG(listener.reset(new CmdHttpListener(IOAddress(SERVER_ADDRESS),
-                                                               SERVER_PORT, num_threads)));
-        ASSERT_TRUE(listener);
+        ASSERT_NO_THROW_LOG(listener_.reset(new CmdHttpListener(IOAddress(SERVER_ADDRESS),
+                                                                SERVER_PORT, num_threads)));
+        ASSERT_TRUE(listener_);
 
-        // Start it and verify it is listening.
-        ASSERT_NO_THROW_LOG(listener->start());
-        ASSERT_TRUE(listener->isListening());
-        EXPECT_EQ(listener->getThreadCount(), num_threads);
+        // Start it and verify it is running.
+        ASSERT_NO_THROW_LOG(listener_->start());
+        ASSERT_TRUE(listener_->isRunning());
+        EXPECT_EQ(listener_->getThreadCount(), num_threads);
 
         // Maps the number of clients served by a given thread-id.
         std::map<std::string, int> clients_per_thread;
@@ -367,9 +403,9 @@ public:
         ASSERT_NO_FATAL_FAILURE(runIOService());
 
         // Stop the listener and then verify it has stopped.
-        ASSERT_NO_THROW_LOG(listener->stop());
-        ASSERT_FALSE(listener->isListening());
-        EXPECT_EQ(listener->getThreadCount(), 0);
+        ASSERT_NO_THROW_LOG(listener_->stop());
+        ASSERT_TRUE(listener_->isStopped());
+        EXPECT_EQ(listener_->getThreadCount(), 0);
 
         // Iterate over the clients, checking their outcomes.
         size_t total_responses = 0;
@@ -462,6 +498,187 @@ public:
         }
     }
 
+    /// @brief Pauses and resumes a CmdHttpListener while it processes command
+    /// requests.
+    ///
+    /// This function command will create a CmdHttpListener
+    /// with the given number of threads, initiates the given
+    /// number of clients, each requesting the "thread" command,
+    /// and then iteratively runs the test's IOService until all
+    /// the clients have received their responses or an error occurs.
+    /// It will pause and resume the listener at intervals governed
+    /// by the given number of pauses.
+    ///
+    /// @param num_threads - the number of threads the CmdHttpListener
+    /// should use. Must be greater than 0.
+    /// @param num_clients - the number of clients that should issue the
+    /// thread command.  Each client is used to carry out a single thread
+    /// command request.  Must be greater than 0.
+    /// @param num_pauses Desired number of times the listener should be
+    /// paused during the test. Must be greater than 0.
+    void workPauseAndResume(size_t num_threads, size_t num_clients,
+                            size_t num_pauses) {
+        // First we makes sure the parameter rules apply.
+        ASSERT_TRUE(num_threads);
+        ASSERT_TRUE(num_clients);
+        ASSERT_TRUE(num_pauses);
+        num_threads_ = num_threads;
+        num_clients_ = num_clients;
+
+        // Register the thread command handler.
+        CommandMgr::instance().registerCommand("thread",
+                                               std::bind(&CmdHttpListenerTest
+                                                         ::simpleCommandHandler,
+                                                         this, ph::_1, ph::_2));
+
+        // Create a listener with prescribed number of threads.
+        ASSERT_NO_THROW_LOG(listener_.reset(new CmdHttpListener(IOAddress(SERVER_ADDRESS),
+                                                                SERVER_PORT, num_threads)));
+        ASSERT_TRUE(listener_);
+
+        // Start it and verify it is running.
+        ASSERT_NO_THROW_LOG(listener_->start());
+        ASSERT_TRUE(listener_->isRunning());
+        EXPECT_EQ(listener_->getThreadCount(), num_threads);
+
+        // Initiate the prescribed number of command requests.
+        num_in_progress_ = 0;
+        for (auto i = 0; clients_.size() < num_clients; ++i) {
+            ASSERT_NO_THROW_LOG(startThreadCommand());
+        }
+
+        // Now we run the client-side IOService until all requests are done,
+        // errors occur or the test times out.  We'll pause and resume the
+        // number of times given by num_pauses.
+        size_t num_done = 0;
+        size_t total_requests = clients_.size();
+        while (num_done < total_requests) {
+            // Calculate how many more requests to process before we pause again.
+            // We divide the number of outstanding requests by the number of pauses
+            // and stop after we've done at least that many more requests.
+            size_t request_limit = (pause_cnt_ < num_pauses ?
+                                    (num_done + ((total_requests - num_done) / num_pauses))
+                                     : total_requests);
+
+            // Run test IOService until we hit the limit.
+            runIOService(request_limit);
+
+            // If we've done all our pauses we should be through.
+            if (pause_cnt_ == num_pauses) {
+                break;
+            }
+
+            // Pause the client.
+            ASSERT_NO_THROW(listener_->pause());
+            ASSERT_TRUE(listener_->isPaused());
+            ++pause_cnt_;
+
+            // Check our progress.
+            num_done = 0;
+            for (auto const& client : clients_) {
+                if (client->receiveDone()) {
+                    ++num_done;
+                }
+            }
+
+            // We should completed at least as many as our
+            // target limit.
+            ASSERT_GE(num_done, request_limit);
+
+            // Resume the listener.
+            ASSERT_NO_THROW(listener_->resume());
+            ASSERT_TRUE(listener_->isRunning());
+        }
+
+        // Stop the listener and then verify it has stopped.
+        ASSERT_NO_THROW_LOG(listener_->stop());
+        ASSERT_TRUE(listener_->isStopped());
+        EXPECT_EQ(listener_->getThreadCount(), 0);
+
+        // Iterate over the clients, checking their outcomes.
+        size_t total_responses = 0;
+        for (auto const& client : clients_) {
+            // Client should have completed its receive successfully.
+            ASSERT_TRUE(client->receiveDone());
+
+            // Client response should not be empty.
+            HttpResponsePtr hr;
+            std::string response_str = client->getResponse();
+            ASSERT_FALSE(response_str.empty());
+
+            // Parse the response into an HttpResponse.
+            ASSERT_NO_THROW_LOG(hr = parseResponse(client->getResponse()));
+
+            // Now we walk the element tree to get the response data.  It should look
+            // this:
+            //
+            //  [ {
+            //       "arguments": { "client-ptr": "xxxxx",
+            //                      "thread-id": "zzzzz" },
+            //       "result": 0
+            //  } ]
+            //
+            // First we turn it into an Element tree.
+            std::string body_str = hr->getBody();
+            ConstElementPtr body;
+            ASSERT_NO_THROW_LOG(body = Element::fromJSON(hr->getBody()));
+
+            // Outermost is a list, since we're emulating agent responses.
+            ASSERT_EQ(body->getType(), Element::list);
+            ASSERT_EQ(body->size(), 1);
+
+            // Answer should be a map containing "arguments" and "results".
+            ConstElementPtr answer = body->get(0);
+            ASSERT_EQ(answer->getType(), Element::map);
+
+            // "result" should be 0.
+            ConstElementPtr result = answer->get("result");
+            ASSERT_TRUE(result);
+            ASSERT_EQ(result->getType(), Element::integer);
+            ASSERT_EQ(result->intValue(), 0);
+
+            // "arguments" is a map containing "client-ptr" and "thread-id".
+            ConstElementPtr arguments = answer->get("arguments");
+            ASSERT_TRUE(arguments);
+            ASSERT_EQ(arguments->getType(), Element::map);
+
+            // "client-ptr" is a string.
+            ConstElementPtr client_ptr = arguments->get("client-ptr");
+            ASSERT_TRUE(client_ptr);
+            ASSERT_EQ(client_ptr->getType(), Element::string);
+
+            // "thread-id" is a string.
+            ConstElementPtr thread_id = arguments->get("thread-id");
+            ASSERT_TRUE(thread_id);
+            ASSERT_EQ(thread_id->getType(), Element::string);
+            std::string thread_id_str = thread_id->stringValue();
+
+            // Make sure the response received was for this client.
+            std::stringstream ss;
+            ss << client;
+            ASSERT_EQ(client_ptr->stringValue(), ss.str());
+
+            ++total_responses;
+        }
+
+        // We should have responses for all our clients.
+        EXPECT_EQ(total_responses, num_clients);
+
+        // We should have had the expected number of pauses.
+        if (!num_pauses) {
+            ASSERT_EQ(pause_cnt_, 0);
+        } else {
+            // We allow a range on pauses of +-1.
+            ASSERT_TRUE((num_pauses - 1) <= pause_cnt_ &&
+                        (pause_cnt_ <= (num_pauses + 1)))
+                        << " num_pauses: " << num_pauses
+                        << ", pause_cnt_" << pause_cnt_;
+        }
+    }
+
+    /// @brief CmdHttpListener instance under test.
+    CmdHttpListenerPtr listener_;
+
     /// @brief IO service used in drive the test and test clients.
     IOService io_service_;
 
@@ -499,104 +716,134 @@ public:
 
     /// @brief Condition variable used to coordinate threads.
     std::condition_variable cv_;
+
+    /// @brief Number of times client has been paused during the test.
+    size_t pause_cnt_;
 };
 
-/// Verifies the construction, starting, stopping, and destruction
-/// of CmdHttpListener.
+/// Verifies the construction, starting, stopping, pausing, resuming,
+/// and destruction of CmdHttpListener.
 TEST_F(CmdHttpListenerTest, basics) {
     // Make sure multi-threading is off.
     MultiThreadingMgr::instance().setMode(false);
-    CmdHttpListenerPtr listener;
     asiolink::IOAddress address(SERVER_ADDRESS);
     uint16_t port = SERVER_PORT;
 
     // Make sure we can create one.
-    ASSERT_NO_THROW_LOG(listener.reset(new CmdHttpListener(address, port)));
-    ASSERT_TRUE(listener);
+    ASSERT_NO_THROW_LOG(listener_.reset(new CmdHttpListener(address, port)));
+    ASSERT_TRUE(listener_);
 
     // Verify the getters do what we expect.
-    EXPECT_EQ(listener->getAddress(), address);
-    EXPECT_EQ(listener->getPort(), port);
-    EXPECT_EQ(listener->getThreadPoolSize(), 1);
+    EXPECT_EQ(listener_->getAddress(), address);
+    EXPECT_EQ(listener_->getPort(), port);
+    EXPECT_EQ(listener_->getThreadPoolSize(), 1);
 
-    // It should not be listening and have no threads.
-    EXPECT_FALSE(listener->isListening());
-    EXPECT_EQ(listener->getThreadCount(), 0);
+    // It should not have an IOService, should not be listening and
+    // should have no threads.
+    ASSERT_FALSE(listener_->getThreadIOService());
+    EXPECT_TRUE(listener_->isStopped());
+    EXPECT_EQ(listener_->getThreadCount(), 0);
 
     // Verify that we cannot start it when multi-threading is disabled.
     ASSERT_FALSE(MultiThreadingMgr::instance().getMode());
-    ASSERT_THROW_MSG(listener->start(), InvalidOperation,
+    ASSERT_THROW_MSG(listener_->start(), InvalidOperation,
                      "CmdHttpListener cannot be started"
                      " when multi-threading is disabled");
 
     // It should still not be listening and have no threads.
-    EXPECT_FALSE(listener->isListening());
-    EXPECT_EQ(listener->getThreadCount(), 0);
+    EXPECT_TRUE(listener_->isStopped());
+    EXPECT_EQ(listener_->getThreadCount(), 0);
 
     // Enable multi-threading.
     MultiThreadingMgr::instance().setMode(true);
 
     // Make sure we can start it and it's listening with 1 thread.
-    ASSERT_NO_THROW_LOG(listener->start());
-    ASSERT_TRUE(listener->isListening());
-    EXPECT_EQ(listener->getThreadCount(), 1);
+    ASSERT_NO_THROW_LOG(listener_->start());
+    ASSERT_TRUE(listener_->isRunning());
+    EXPECT_EQ(listener_->getThreadCount(), 1);
+    ASSERT_TRUE(listener_->getThreadIOService());
+    EXPECT_FALSE(listener_->getThreadIOService()->stopped());
 
     // Trying to start it again should fail.
-    ASSERT_THROW_MSG(listener->start(), InvalidOperation,
-                     "CmdHttpListener is already listening!");
+    ASSERT_THROW_MSG(listener_->start(), InvalidOperation,
+                     "CmdHttpListener already started!");
 
     // Stop it and verify we're no longer listening.
-    ASSERT_NO_THROW_LOG(listener->stop());
-    ASSERT_FALSE(listener->isListening());
-    EXPECT_EQ(listener->getThreadCount(), 0);
+    ASSERT_NO_THROW_LOG(listener_->stop());
+    ASSERT_TRUE(listener_->isStopped());
+    EXPECT_EQ(listener_->getThreadCount(), 0);
+    ASSERT_FALSE(listener_->getThreadIOService());
 
     // Make sure we can call stop again without problems.
-    ASSERT_NO_THROW_LOG(listener->stop());
+    ASSERT_NO_THROW_LOG(listener_->stop());
 
     // We should be able to restart it.
-    ASSERT_NO_THROW_LOG(listener->start());
-    ASSERT_TRUE(listener->isListening());
-    EXPECT_EQ(listener->getThreadCount(), 1);
+    ASSERT_NO_THROW_LOG(listener_->start());
+    ASSERT_TRUE(listener_->isRunning());
+    EXPECT_EQ(listener_->getThreadCount(), 1);
+    ASSERT_TRUE(listener_->getThreadIOService());
+    EXPECT_FALSE(listener_->getThreadIOService()->stopped());
 
     // Destroying it should also stop it.
     // If the test timeouts we know it didn't!
-    ASSERT_NO_THROW_LOG(listener.reset());
+    ASSERT_NO_THROW_LOG(listener_.reset());
 
     // Verify we can construct with more than one thread.
-    ASSERT_NO_THROW_LOG(listener.reset(new CmdHttpListener(address, port, 4)));
-    ASSERT_NO_THROW_LOG(listener->start());
-    EXPECT_EQ(listener->getAddress(), address);
-    EXPECT_EQ(listener->getPort(), port);
-    EXPECT_EQ(listener->getThreadPoolSize(), 4);
-    ASSERT_TRUE(listener->isListening());
-    EXPECT_EQ(listener->getThreadCount(), 4);
+    ASSERT_NO_THROW_LOG(listener_.reset(new CmdHttpListener(address, port, 4)));
+    ASSERT_NO_THROW_LOG(listener_->start());
+    EXPECT_EQ(listener_->getAddress(), address);
+    EXPECT_EQ(listener_->getPort(), port);
+    EXPECT_EQ(listener_->getThreadCount(), 4);
+    EXPECT_EQ(listener_->getThreadPoolSize(), 4);
+    ASSERT_TRUE(listener_->isRunning());
+    ASSERT_TRUE(listener_->getThreadIOService());
+    EXPECT_FALSE(listener_->getThreadIOService()->stopped());
+
+    // Verify we can pause it.  We should still be listening, threads intact,
+    // IOService stopped, state set to PAUSED.
+    ASSERT_NO_THROW_LOG(listener_->pause());
+    ASSERT_TRUE(listener_->isPaused());
+    EXPECT_EQ(listener_->getThreadCount(), 4);
+    EXPECT_EQ(listener_->getThreadPoolSize(), 4);
+    ASSERT_TRUE(listener_->getThreadIOService());
+    EXPECT_TRUE(listener_->getThreadIOService()->stopped());
+
+    // Verify we can resume it.
+    ASSERT_NO_THROW_LOG(listener_->resume());
+    ASSERT_TRUE(listener_->isRunning());
+    EXPECT_EQ(listener_->getThreadCount(), 4);
+    EXPECT_EQ(listener_->getThreadPoolSize(), 4);
+    ASSERT_TRUE(listener_->getThreadIOService());
+    EXPECT_FALSE(listener_->getThreadIOService()->stopped());
 
     // Stop it and verify we're no longer listening.
-    ASSERT_NO_THROW_LOG(listener->stop());
-    ASSERT_FALSE(listener->isListening());
-    EXPECT_EQ(listener->getThreadCount(), 0);
+    ASSERT_NO_THROW_LOG(listener_->stop());
+    ASSERT_TRUE(listener_->isStopped());
+    EXPECT_EQ(listener_->getThreadCount(), 0);
+    EXPECT_EQ(listener_->getThreadPoolSize(), 4);
+    ASSERT_FALSE(listener_->getThreadIOService());
+    EXPECT_TRUE(listener_->isStopped());
 }
 
-
 // This test verifies that an HTTP connection can be established and used to
 // transmit an HTTP request and receive the response.
 TEST_F(CmdHttpListenerTest, basicListenAndRespond) {
 
     // Create a listener with 1 thread.
-    CmdHttpListenerPtr listener;
-    ASSERT_NO_THROW_LOG(listener.reset(new CmdHttpListener(IOAddress(SERVER_ADDRESS),
+    ASSERT_NO_THROW_LOG(listener_.reset(new CmdHttpListener(IOAddress(SERVER_ADDRESS),
                                                            SERVER_PORT)));
-    ASSERT_TRUE(listener);
+    ASSERT_TRUE(listener_);
 
     // Start the listener and verify it's listening with 1 thread.
-    ASSERT_NO_THROW_LOG(listener->start());
-    ASSERT_TRUE(listener->isListening());
-    EXPECT_EQ(listener->getThreadCount(), 1);
+    ASSERT_NO_THROW_LOG(listener_->start());
+    ASSERT_TRUE(listener_->isRunning());
+    EXPECT_EQ(listener_->getThreadCount(), 1);
 
     // Now let's send a "foo" command.  This should create a client, connect
     // to our listener, post our request and retrieve our reply.
     ASSERT_NO_THROW(startRequest("{\"command\": \"foo\"}"));
-    ASSERT_EQ(1, clients_.size());
+    ++num_clients_;
+    ASSERT_EQ(num_clients_, clients_.size());
     ASSERT_NO_THROW(runIOService());
     TestHttpClientPtr client = clients_.front();
     ASSERT_TRUE(client);
@@ -614,7 +861,8 @@ TEST_F(CmdHttpListenerTest, basicListenAndRespond) {
                                                       this, ph::_1, ph::_2));
     // Try posting the foo command again.
     ASSERT_NO_THROW(startRequest("{\"command\": \"foo\"}"));
-    ASSERT_EQ(2, clients_.size());
+    ++num_clients_;
+    ASSERT_EQ(num_clients_, clients_.size());
     ASSERT_NO_THROW(runIOService());
     client = clients_.back();
     ASSERT_TRUE(client);
@@ -626,13 +874,13 @@ TEST_F(CmdHttpListenerTest, basicListenAndRespond) {
     EXPECT_EQ(hr->getBody(), "[ { \"arguments\": [ \"bar\" ], \"result\": 0 } ]");
 
     // Make sure the listener is still listening.
-    ASSERT_TRUE(listener->isListening());
-    EXPECT_EQ(listener->getThreadCount(), 1);
+    ASSERT_TRUE(listener_->isRunning());
+    EXPECT_EQ(listener_->getThreadCount(), 1);
 
     // Stop the listener then verify it has stopped.
-    ASSERT_NO_THROW_LOG(listener->stop());
-    ASSERT_FALSE(listener->isListening());
-    EXPECT_EQ(listener->getThreadCount(), 0);
+    ASSERT_NO_THROW_LOG(listener_->stop());
+    ASSERT_TRUE(listener_->isStopped());
+    EXPECT_EQ(listener_->getThreadCount(), 0);
 }
 
 // Now we'll run some permutations of the number of listener threads
@@ -680,4 +928,13 @@ TEST_F(CmdHttpListenerTest, sixByEighteen) {
     threadListenAndRespond(num_threads, num_clients);
 }
 
+// Pauses and resumes the listener while it is processing
+// requests.
+TEST_F(CmdHttpListenerTest, pauseAndResume) {
+    size_t num_threads = 6;
+    size_t num_clients = 18;
+    size_t num_pauses = 3;
+    workPauseAndResume(num_threads, num_clients, num_pauses);
+}
+
 } // end of anonymous namespace
