@@ -27,6 +27,8 @@
 #include <map>
 #include <mutex>
 #include <queue>
+#include <thread>
+
 
 using namespace isc;
 using namespace isc::asiolink;
@@ -90,10 +92,10 @@ typedef boost::shared_ptr<ConnectionPool> ConnectionPoolPtr;
 /// @brief Client side HTTP connection to the server.
 ///
 /// Each connection is established with a unique destination identified by the
-/// specified URL. Multiple requests to the same destination can be sent over
-/// the same connection, if the connection is persistent. If the server closes
-/// the TCP connection (e.g. after sending a response), the connection can
-/// be re-established (using the same @c Connection object).
+/// specified URL and TLS context. Multiple requests to the same destination
+/// can be sent over the same connection, if the connection is persistent.
+/// If the server closes the TCP connection (e.g. after sending a response),
+/// the connection is closed.
 ///
 /// If new request is created while the previous request is still in progress,
 /// the new request is stored in the FIFO queue. The queued requests to the
@@ -152,7 +154,22 @@ public:
     /// @brief Checks if a transaction has been initiated over this connection.
     ///
     /// @return true if transaction has been initiated, false otherwise.
-    bool isTransactionOngoing() const;
+    bool isTransactionOngoing() const {
+        return (started_);
+    }
+
+    /// @brief Checks if the socket has been closed.
+    ///
+    /// @return true if the socket has been closed.
+    bool isClosed() const {
+        return (closed_);
+    }
+
+    /// @brief Checks if the peer has closed the idle socket at its side.
+    ///
+    /// If the socket is open but is not usable the peer has closed
+    /// the socket at its side so we close it.
+    void isClosedByPeer();
 
     /// @brief Checks if a socket descriptor belongs to this connection.
     ///
@@ -213,6 +230,14 @@ private:
     /// Should be called in a thread safe context.
     void closeInternal();
 
+    /// @brief Checks if the peer has closed the socket at its side.
+    ///
+    /// Should be called in a thread safe context.
+    ///
+    /// If the socket is open but is not usable the peer has closed
+    /// the socket at its side so we close it.
+    void isClosedByPeerInternal();
+
     /// @brief Checks and logs if premature transaction timeout is suspected.
     ///
     /// Should be called in a thread safe context.
@@ -291,6 +316,8 @@ private:
 
     /// @brief Asynchronously performs the TLS handshake.
     ///
+    /// The TLS handshake is performed once on TLS sockets.
+    ///
     /// @param transid Current transaction id.
     void doHandshake(const uint64_t transid);
 
@@ -380,6 +407,9 @@ private:
     /// @brief URL for this connection.
     Url url_;
 
+    /// @brief TLS context for this connection.
+    TlsContextPtr tls_context_;
+
     /// @brief TCP socket to be used for this connection.
     std::unique_ptr<TCPSocket<SocketCallback> > tcp_socket_;
 
@@ -419,6 +449,12 @@ private:
     /// @brief Flag to indicate that a transaction is running.
     std::atomic<bool> started_;
 
+    /// @brief Flag to indicate that the TLS handshake has to be performed.
+    std::atomic<bool> need_handshake_;
+
+    /// @brief Flag to indicate that the socket was closed.
+    std::atomic<bool> closed_;
+
     /// @brief Mutex to protect the internal state.
     std::mutex mutex_;
 };
@@ -443,7 +479,7 @@ public:
     /// @param max_url_connections maximum number of concurrent
     /// connections allowed per URL.
     explicit ConnectionPool(IOService& io_service, size_t max_url_connections)
-        : io_service_(io_service), destinations_(), mutex_(),
+        : io_service_(io_service), destinations_(), pool_mutex_(),
           max_url_connections_(max_url_connections) {
     }
 
@@ -454,18 +490,31 @@ public:
         closeAll();
     }
 
-    /// @brief Process next queued request for the given URL.
+    /// @brief Process next queued request for the given URL and TLS context.
     ///
     /// @param url URL for which next queued request should be processed.
-    void processNextRequest(const Url& url) {
+    /// @param tls_context TLS context for which next queued request
+    /// should be processed.
+    void processNextRequest(const Url& url, const TlsContextPtr& tls_context) {
         if (MultiThreadingMgr::instance().getMode()) {
-            std::lock_guard<std::mutex> lk(mutex_);
-            return (processNextRequestInternal(url));
+            std::lock_guard<std::mutex> lk(pool_mutex_);
+            return (processNextRequestInternal(url, tls_context));
         } else {
-            return (processNextRequestInternal(url));
+            return (processNextRequestInternal(url, tls_context));
         }
     }
 
+    /// @brief Schedule processing of next queued request.
+    ///
+    /// @param url URL for which next queued request should be processed.
+    /// @param tls_context TLS context for which next queued request
+    /// should be processed.
+    void postProcessNextRequest(const Url& url,
+                                const TlsContextPtr& tls_context) {
+        io_service_.post(std::bind(&ConnectionPool::processNextRequest,
+                                   shared_from_this(), url, tls_context));
+    }
+
     /// @brief Queue next request for sending to the server.
     ///
     /// A new transaction is started immediately, if there is no other request
@@ -496,7 +545,7 @@ public:
                       const HttpClient::HandshakeHandler& handshake_callback,
                       const HttpClient::CloseHandler& close_callback) {
         if (MultiThreadingMgr::instance().getMode()) {
-            std::lock_guard<std::mutex> lk(mutex_);
+            std::lock_guard<std::mutex> lk(pool_mutex_);
             return (queueRequestInternal(url, tls_context, request, response,
                                          request_timeout, request_callback,
                                          connect_callback, handshake_callback,
@@ -513,7 +562,7 @@ public:
     /// the connection pool.
     void closeAll() {
         if (MultiThreadingMgr::instance().getMode()) {
-            std::lock_guard<std::mutex> lk(mutex_);
+            std::lock_guard<std::mutex> lk(pool_mutex_);
             closeAllInternal();
         } else {
             closeAllInternal();
@@ -534,7 +583,7 @@ public:
     /// @param socket_fd socket descriptor to check
     void closeIfOutOfBand(int socket_fd) {
         if (MultiThreadingMgr::instance().getMode()) {
-            std::lock_guard<std::mutex> lk(mutex_);
+            std::lock_guard<std::mutex> lk(pool_mutex_);
             closeIfOutOfBandInternal(socket_fd);
         } else {
             closeIfOutOfBandInternal(socket_fd);
@@ -543,23 +592,35 @@ public:
 
 private:
 
-    /// @brief Process next queued request for the given URL.
+    /// @brief Process next queued request for the given URL and TLS context.
     ///
     /// This method should be called in a thread safe context.
     ///
     /// @param url URL for which next queued request should be retrieved.
-    void processNextRequestInternal(const Url& url) {
+    /// @param tls_context TLS context for which next queued request
+    /// should be processed.
+    void processNextRequestInternal(const Url& url,
+                                    const TlsContextPtr& tls_context) {
         // Check if there is a queue for this URL. If there is no queue, there
         // is no request queued either.
-        DestinationPtr destination = findDestination(url);
+        DestinationPtr destination = findDestination(url, tls_context);
         if (destination) {
+            // Remove closed connections.
+            destination->garbageCollectConnections();
             if (!destination->queueEmpty()) {
                 // We have at least one queued request. Do we have an
                 // idle connection?
                 ConnectionPtr connection = destination->getIdleConnection();
                 if (!connection) {
-                    // No idle connections, so just return.
-                    return;
+                    // No idle connections.
+                    if (destination->connectionsFull()) {
+                        return;
+                    }
+                    // Room to make another connection with this destination,
+                    // so make one.
+                    connection.reset(new Connection(io_service_, tls_context,
+                                                    shared_from_this(), url));
+                    destination->addConnection(connection);
                 }
 
                 // Dequeue the oldest request and start a transaction for it using
@@ -607,13 +668,15 @@ private:
                               const HttpClient::CloseHandler& close_callback) {
         ConnectionPtr connection;
         // Find the destination for the requested URL.
-        DestinationPtr destination = findDestination(url);
+        DestinationPtr destination = findDestination(url, tls_context);
         if (destination) {
+            // Remove closed connections.
+            destination->garbageCollectConnections();
             // Found it, look for an idle connection.
             connection = destination->getIdleConnection();
         } else {
             // Doesn't exist yet so it's a new destination.
-            destination = addDestination(url);
+            destination = addDestination(url, tls_context);
         }
 
         if (!connection) {
@@ -736,16 +799,22 @@ private:
         HttpClient::CloseHandler close_callback_;
     };
 
+    /// @brief Type of URL and TLS context pairs.
+    typedef std::pair<Url, TlsContextPtr> DestinationDescriptor;
+
     /// @brief Encapsulates connections and requests for a given URL
     class Destination {
     public:
         /// @brief Constructor
         ///
         /// @param url server URL of this destination
+        /// @param tls_context server TLS context of this destination
         /// @param max_connections maximum number of concurrent connections
         /// allowed for in the list URL
-        Destination(Url url, size_t max_connections)
-        : url_(url), max_connections_(max_connections), connections_(), queue_() { }
+        Destination(Url url, TlsContextPtr tls_context, size_t max_connections)
+            : url_(url), tls_context_(tls_context),
+              max_connections_(max_connections), connections_(), queue_() {
+        }
 
         /// @brief Destructor
         ~Destination() {
@@ -786,6 +855,11 @@ private:
         /// @brief Closes all connections and clears the list.
         /// @note This should be called in a thread safe context.
         void closeAllConnections() {
+            // Flush the queue.
+            while (!queue_.empty()) {
+                queue_.pop();
+            }
+
             for (auto const& connection : connections_) {
                 connection->close();
             }
@@ -793,16 +867,55 @@ private:
             connections_.clear();
         }
 
+        /// @brief Removes closed connections.
+        ///
+        /// This method should be called before @ref getIdleConnection.
+        ///
+        /// In a first step it closes not usable idle connections
+        /// (idle means no current transaction and not closed,
+        /// usable means the peer side did not close it at that time).
+        /// In a second step it removes (collects) closed connections.
+        ///
+        /// @note a connection is closed when the transaction is finished
+        /// and the connection is persistent, or when the connection was
+        /// idle and the first step of the garbage collector detects that
+        /// it was closed by peer, so is not usable.
+        ///
+        /// @note there are two races here:
+        ///  - the peer side closes the connection after the first step
+        ///  - a not persistent connection finishes its transaction and
+        ///    closes
+        /// The second race is avoided by setting the closed flag before
+        /// the started flag and by unconditionally posting a process next
+        /// request action.
+        ///
+        /// @note This should be called in a thread safe context.
+        void garbageCollectConnections() {
+            for (auto it = connections_.begin(); it != connections_.end();) {
+                (*it)->isClosedByPeer();
+                if (!(*it)->isClosed()) {
+                    ++it;
+                } else {
+                    it = connections_.erase(it);
+                }
+            }
+        }
+
         /// @brief Finds the first idle connection.
         ///
         /// Iterates over the existing connections and returns the
-        /// first connection which is not currently in a transaction.
+        /// first connection which is not currently in a transaction and
+        /// is not closed.
+        ///
+        /// @note @ref garbageCollectConnections should be called before.
+        /// This removes connections which were closed at that time.
         ///
         /// @return The first idle connection or an empty pointer if
-        /// all connections are busy.
+        /// all connections are busy or closed.
         ConnectionPtr getIdleConnection() {
             for (auto const& connection : connections_) {
-                if (!connection->isTransactionOngoing()) {
+                if (!connection->isTransactionOngoing() &&
+                    !connection->isClosed()) {
                     return (connection);
                 }
             }
@@ -854,13 +967,6 @@ private:
             return (max_connections_);
         }
 
-        /// @brief Fetches the URL.
-        ///
-        /// @return the URL.
-        const Url& getUrl() const {
-            return (url_);
-        }
-
         /// @brief Indicates if request queue is empty.
         ///
         /// @return true if there are no requests queued.
@@ -892,6 +998,9 @@ private:
         /// @brief URL supported by this destination.
         Url url_;
 
+        /// @brief TLS context to use with this destination.
+        TlsContextPtr tls_context_;
+
         /// @brief Maximum number of concurrent connections for this destination.
         size_t max_connections_;
 
@@ -905,27 +1014,34 @@ private:
     /// @brief Pointer to a Destination.
     typedef boost::shared_ptr<Destination> DestinationPtr;
 
-    /// @brief Creates a new destination for the given URL.
+    /// @brief Creates a new destination for the given URL and TLS context.
     ///
     /// @param url URL of the new destination.
+    /// @param tls_context TLS context for the new destination.
     ///
     /// @return Pointer to the newly created destination.
     /// @note Must be called from within a thread-safe context.
-    DestinationPtr addDestination(const Url& url) {
-        DestinationPtr destination(new Destination(url, max_url_connections_));
-        destinations_[url] = destination;
+    DestinationPtr addDestination(const Url& url,
+                                  const TlsContextPtr& tls_context) {
+        const DestinationDescriptor& desc = std::make_pair(url, tls_context);
+        DestinationPtr destination(new Destination(url, tls_context,
+                                                   max_url_connections_));
+        destinations_[desc] = destination;
         return (destination);
     }
 
-    /// @brief Fetches a destination by URL
+    /// @brief Fetches a destination by URL and TLS context.
     ///
     /// @param url URL of the destination desired.
+    /// @param tls_context TLS context for the destination desired.
     ///
     /// @return pointer the desired destination, empty pointer
     /// if the destination does not exist.
     /// @note Must be called from within a thread-safe context.
-    DestinationPtr findDestination(const Url& url) const {
-        auto it = destinations_.find(url);
+    DestinationPtr findDestination(const Url& url,
+                                   const TlsContextPtr& tls_context) const {
+        const DestinationDescriptor& desc = std::make_pair(url, tls_context);
+        auto it = destinations_.find(desc);
         if (it != destinations_.end()) {
             return (it->second);
         }
@@ -933,16 +1049,21 @@ private:
         return (DestinationPtr());
     }
 
-    /// @brief Removes a destination by URL
+    /// @brief Removes a destination by URL and TLS context.
     ///
     /// Closes all of the destination's connections and
     /// discards all of its queued requests while removing
     /// the destination from the list of known destinations.
     ///
+    /// @note not used yet.
+    ///
     /// @param url URL of the destination to be removed.
+    /// @param tls_context TLS context for the destination to be removed.
     /// @note Must be called from within a thread-safe context.
-    void removeDestination(const Url& url) {
-        auto it = destinations_.find(url);
+    void removeDestination(const Url& url,
+                           const TlsContextPtr& tls_context) {
+        const DestinationDescriptor& desc = std::make_pair(url, tls_context);
+        auto it = destinations_.find(desc);
         if (it != destinations_.end()) {
             it->second->closeAllConnections();
             destinations_.erase(it);
@@ -952,13 +1073,13 @@ private:
     /// @brief A reference to the IOService that drives socket IO.
     IOService& io_service_;
 
-    /// @brief Map of Destinations by URL.
-    std::map<Url, DestinationPtr> destinations_;
+    /// @brief Map of Destinations by URL and TLS context.
+    std::map<DestinationDescriptor, DestinationPtr> destinations_;
 
     /// @brief Mutex to protect the internal state.
-    std::mutex mutex_;
+    std::mutex pool_mutex_;
 
-    /// @brief Maximum number of connections per URL.
+    /// @brief Maximum number of connections per URL and TLS context.
     size_t max_url_connections_;
 };
 
@@ -966,15 +1087,18 @@ Connection::Connection(IOService& io_service,
                        const TlsContextPtr& tls_context,
                        const ConnectionPoolPtr& conn_pool,
                        const Url& url)
-    : conn_pool_(conn_pool), url_(url), tcp_socket_(), tls_socket_(),
-      timer_(io_service), current_request_(), current_response_(),
-      parser_(), current_callback_(), buf_(), input_buf_(),
-      current_transid_(0), close_callback_(), started_(false) {
+    : conn_pool_(conn_pool), url_(url), tls_context_(tls_context),
+      tcp_socket_(), tls_socket_(), timer_(io_service),
+      current_request_(), current_response_(), parser_(),
+      current_callback_(), buf_(), input_buf_(), current_transid_(0),
+      close_callback_(), started_(false), need_handshake_(false),
+      closed_(false) {
     if (!tls_context) {
         tcp_socket_.reset(new asiolink::TCPSocket<SocketCallback>(io_service));
     } else {
         tls_socket_.reset(new asiolink::TLSSocket<SocketCallback>(io_service,
                                                                   tls_context));
+        need_handshake_ = true;
     }
 }
 
@@ -1013,6 +1137,49 @@ Connection::closeCallback(const bool clear) {
     }
 }
 
+void
+Connection::isClosedByPeer() {
+    // This method applies only to idle connections.
+    if (started_ || closed_) {
+        return;
+    }
+    // This code was guarded by a lock so keep this.
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(mutex_);
+        isClosedByPeerInternal();
+    } else {
+        isClosedByPeerInternal();
+    }
+}
+
+void
+Connection::isClosedByPeerInternal() {
+    // If the socket is open we check if it is possible to transmit
+    // the data over this socket by reading from it with message
+    // peeking. If the socket is not usable, we close it and then
+    // re-open it. There is a narrow window of time between checking
+    // the socket usability and actually transmitting the data over
+    // this socket, when the peer may close the connection. In this
+    // case we'll need to re-transmit but we don't handle it here.
+    if (tcp_socket_) {
+        if (tcp_socket_->getASIOSocket().is_open() &&
+            !tcp_socket_->isUsable()) {
+            closeCallback();
+            closed_ = true;
+            tcp_socket_->close();
+        }
+    } else if (tls_socket_) {
+        if (tls_socket_->getASIOSocket().is_open() &&
+            !tls_socket_->isUsable()) {
+            closeCallback();
+            closed_ = true;
+            tls_socket_->close();
+        }
+    } else {
+        isc_throw(Unexpected, "internal error: can't find the sending socket");
+    }
+}
+
 void
 Connection::doTransaction(const HttpRequestPtr& request,
                           const HttpResponsePtr& response,
@@ -1056,29 +1223,6 @@ Connection::doTransactionInternal(const HttpRequestPtr& request,
 
         buf_ = request->toString();
 
-        // If the socket is open we check if it is possible to transmit the data
-        // over this socket by reading from it with message peeking. If the socket
-        // is not usable, we close it and then re-open it. There is a narrow window of
-        // time between checking the socket usability and actually transmitting the
-        // data over this socket, when the peer may close the connection. In this
-        // case we'll need to re-transmit but we don't handle it here.
-        if (tcp_socket_) {
-            if (tcp_socket_->getASIOSocket().is_open() &&
-                !tcp_socket_->isUsable()) {
-                closeCallback();
-                tcp_socket_->close();
-            }
-        } else if (tls_socket_) {
-            if (tls_socket_->getASIOSocket().is_open() &&
-                !tls_socket_->isUsable()) {
-                closeCallback();
-                tls_socket_->close();
-            }
-        } else {
-            isc_throw(Unexpected,
-                      "internal error: can't find the sending socket");
-        }
-
         LOG_DEBUG(http_logger, isc::log::DBGLVL_TRACE_DETAIL,
                   HTTP_CLIENT_REQUEST_SEND)
             .arg(request->toBriefString())
@@ -1113,8 +1257,7 @@ Connection::doTransactionInternal(const HttpRequestPtr& request,
         }
 
         // Should never reach this point.
-        isc_throw(Unexpected,
-                  "internal error: can't find a socket to open");
+        isc_throw(Unexpected, "internal error: can't find a socket to open");
 
     } catch (const std::exception& ex) {
         // Re-throw with the expected exception type.
@@ -1137,6 +1280,7 @@ Connection::closeInternal() {
     // Pass in true to discard the callback.
     closeCallback(true);
 
+    closed_ = true;
     timer_.cancel();
     if (tcp_socket_) {
         tcp_socket_->close();
@@ -1148,11 +1292,6 @@ Connection::closeInternal() {
     resetState();
 }
 
-bool
-Connection::isTransactionOngoing() const {
-    return (started_);
-}
-
 bool
 Connection::isMySocket(int socket_fd) const {
     if (tcp_socket_) {
@@ -1265,25 +1404,22 @@ Connection::terminateInternal(const boost::system::error_code& ec,
         } catch (...) {
         }
 
-        // If we're not requesting connection persistence or the connection has timed out,
-        // we should close the socket. We're going to reconnect for the next transaction.
-        if (!current_request_->isPersistent() || (ec == boost::asio::error::timed_out)) {
+        // If we're not requesting connection persistence or the
+        // connection has timed out, we should close the socket.
+        if (!closed_ &&
+            (!current_request_->isPersistent() ||
+             (ec == boost::asio::error::timed_out))) {
             closeInternal();
         }
 
         resetState();
     }
 
-    // Check if there are any requests queued for this connection and start
+    // Check if there are any requests queued for this destination and start
     // another transaction if there is at least one.
     ConnectionPoolPtr conn_pool = conn_pool_.lock();
     if (conn_pool) {
-        if (MultiThreadingMgr::instance().getMode()) {
-            UnlockGuard<std::mutex> lock(mutex_);
-            conn_pool->processNextRequest(url_);
-        } else {
-            conn_pool->processNextRequest(url_);
-        }
+        conn_pool->postProcessNextRequest(url_, tls_context_);
     }
 }
 
@@ -1297,8 +1433,8 @@ Connection::scheduleTimer(const long request_timeout) {
 
 void
 Connection::doHandshake(const uint64_t transid) {
-    // Skip the handshake if the socket is not a TLS one.
-    if (!tls_socket_) {
+    // Skip the handshake if it is not needed.
+    if (!need_handshake_) {
         doSend(transid);
         return;
     }
@@ -1422,6 +1558,7 @@ void
 Connection::handshakeCallback(HttpClient::ConnectHandler handshake_callback,
                               const uint64_t transid,
                               const boost::system::error_code& ec) {
+    need_handshake_ = false;
     if (checkPrematureTimeout(transid)) {
         return;
     }
@@ -1601,30 +1738,32 @@ public:
     ///
     /// @param io_service IOService that will drive connection IO in single
     /// threaded mode.  (Currently ignored in multi-threaded mode)
-    ///
     /// @param thread_pool_size maximum number of concurrent threads
-    /// Internally this also sets the maximum number concurrent connections
+    /// Internally this also sets the maximum number of concurrent connections
     /// per URL.
-    HttpClientImpl(IOService& io_service, size_t thread_pool_size = 0) :
-        thread_pool_size_(thread_pool_size) {
+    /// @param defer_thread_start When true, starting of the pool threads is
+    /// deferred until a subsequent call to @ref start(). In this case the
+    /// pool's operational state after construction is STOPPED.  Otherwise,
+    /// the thread pool threads will be created and started, with the
+    /// operational state being RUNNING.  Applicable only when thread-pool size
+    /// is greater than zero.
+    HttpClientImpl(IOService& io_service, size_t thread_pool_size = 0,
+                   bool defer_thread_start = false)
+        : thread_pool_size_(thread_pool_size), thread_pool_() {
         if (thread_pool_size_ > 0) {
             // Create our own private IOService.
             thread_io_service_.reset(new IOService());
 
-            // Create a pool of threads, each calls run on the same, private
-            // io_service instance
-            for (std::size_t i = 0; i < thread_pool_size_; ++i) {
-                boost::shared_ptr<std::thread> thread(new std::thread(std::bind(&IOService::run,
-                                                                      thread_io_service_)));
-                threads_.push_back(thread);
-            }
+            // Create the thread pool.
+            thread_pool_.reset(new HttpThreadPool(thread_io_service_, thread_pool_size_,
+                                                  defer_thread_start));
 
             // Create the connection pool. Note that we use the thread_pool_size
             // as the maximum connections per URL value.
             conn_pool_.reset(new ConnectionPool(*thread_io_service_, thread_pool_size_));
 
             LOG_DEBUG(http_logger, isc::log::DBGLVL_TRACE_BASIC, HTTP_CLIENT_MT_STARTED)
-                     .arg(threads_.size());
+                     .arg(getThreadCount());
         } else {
             // Single-threaded mode: use the caller's IOService,
             // one connection per URL.
@@ -1639,26 +1778,85 @@ public:
         stop();
     }
 
-    /// @brief Close all connections, and if multi-threaded, stop internal IOService
-    /// and the thread pool.
-    void stop() {
-        if (thread_io_service_) {
-            // Stop the private IOService.
-            thread_io_service_->stop();
-
-            // Shutdown the threads.
-            for (auto const& thread : threads_) {
-                thread->join();
-            }
-
-            threads_.clear();
+    /// @brief Starts running the client's thread pool, if multi-threaded.
+    void start() {
+        if (thread_pool_) {
+            thread_pool_->run();
         }
+    }
 
+    /// @brief Close all connections, and if multi-threaded, stops the client's
+    /// thread pool.
+    void stop() {
         // Close all the connections.
         conn_pool_->closeAll();
 
-        // Get rid of the IOService.
-        thread_io_service_.reset();
+        // Stop the thread pool.
+        if (thread_pool_) {
+            thread_pool_->stop();
+        }
+    }
+
+    /// @brief Pauses the client's thread pool.
+    ///
+    /// Suspends thread pool event processing.
+    /// @throw InvalidOperation if the thread pool does not exist.
+    void pause() {
+        if (!thread_pool_) {
+            isc_throw(InvalidOperation, "HttpClient::pause - no thread pool");
+        }
+
+        // Pause the thread pool.
+        thread_pool_->pause();
+    }
+
+    /// @brief Resumes running the client's thread pool.
+    ///
+    /// Resumes thread pool event processing.
+    /// @throw InvalidOperation if the thread pool does not exist.
+    void resume() {
+        if (!thread_pool_) {
+            isc_throw(InvalidOperation, "HttpClient::resume - no thread pool");
+        }
+
+        // Resume running the thread pool.
+        thread_pool_->run();
+    }
+
+    /// @brief Indicates if the thread pool is running.
+    ///
+    /// @return True if the thread pool exists and it is in the RUNNING state,
+    /// false otherwise.
+    bool isRunning() {
+        if (thread_pool_) {
+            return (thread_pool_->isRunning());
+        }
+
+        return (false);
+    }
+
+    /// @brief Indicates if the thread pool is stopped.
+    ///
+    /// @return True if the thread pool exists and it is in the STOPPED state,
+    /// false otherwise.
+    bool isStopped() {
+        if (thread_pool_) {
+            return (thread_pool_->isStopped());
+        }
+
+        return (false);
+    }
+
+    /// @brief Indicates if the thread pool is paused.
+    ///
+    /// @return True if the thread pool exists and it is in the PAUSED state,
+    /// false otherwise.
+    bool isPaused() {
+        if (thread_pool_) {
+            return (thread_pool_->isPaused());
+        }
+
+        return (false);
     }
 
     /// @brief Fetches the internal IOService used in multi-threaded mode.
@@ -1680,25 +1878,30 @@ public:
     ///
     /// @return the number of running threads.
     uint16_t getThreadCount() {
-        return (threads_.size());
+        if (!thread_pool_) {
+            return (0);
+        }
+        return (thread_pool_->getThreadCount());
     }
 
     /// @brief Holds a pointer to the connection pool.
     ConnectionPoolPtr conn_pool_;
 
 private:
+
     /// @brief Maxim number of threads in the thread pool.
     size_t thread_pool_size_;
 
-    /// @brief Pool of threads used to service connections in multi-threaded
-    /// mode.
-    std::vector<boost::shared_ptr<std::thread> > threads_;
-
     /// @brief Pointer to private IOService used in multi-threaded mode.
     asiolink::IOServicePtr thread_io_service_;
+
+    /// @brief Pool of threads used to service connections in multi-threaded
+    /// mode.
+    HttpThreadPoolPtr thread_pool_;
 };
 
-HttpClient::HttpClient(IOService& io_service, size_t thread_pool_size) {
+HttpClient::HttpClient(IOService& io_service, size_t thread_pool_size,
+                       bool defer_thread_start /* = false */) {
     if (thread_pool_size > 0) {
         if (!MultiThreadingMgr::instance().getMode()) {
             isc_throw(InvalidOperation,
@@ -1707,7 +1910,8 @@ HttpClient::HttpClient(IOService& io_service, size_t thread_pool_size) {
         }
     }
 
-    impl_.reset(new HttpClientImpl(io_service, thread_pool_size));
+    impl_.reset(new HttpClientImpl(io_service, thread_pool_size,
+                                   defer_thread_start));
 }
 
 HttpClient::~HttpClient() {
@@ -1754,6 +1958,21 @@ HttpClient::closeIfOutOfBand(int socket_fd)  {
     return (impl_->conn_pool_->closeIfOutOfBand(socket_fd));
 }
 
+void
+HttpClient::start() {
+    impl_->start();
+}
+
+void
+HttpClient::pause() {
+    impl_->pause();
+}
+
+void
+HttpClient::resume() {
+    impl_->resume();
+}
+
 void
 HttpClient::stop() {
     impl_->stop();
@@ -1774,5 +1993,20 @@ HttpClient::getThreadCount() const {
     return (impl_->getThreadCount());
 }
 
+bool
+HttpClient::isRunning() {
+    return (impl_->isRunning());
+}
+
+bool
+HttpClient::isStopped() {
+    return (impl_->isStopped());
+}
+
+bool
+HttpClient::isPaused() {
+    return (impl_->isPaused());
+}
+
 } // end of namespace isc::http
 } // end of namespace isc
